﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Romulus.Nes;
using System.IO;

namespace GearTool
{
    public partial class Form1 : Form
    {
        GearRom rom;
        

        // Set to true if the user cancels the open dialog
        bool abortProgram;

        public Form1() {
            InitializeComponent();

            if (RomDialog.ShowDialog() == DialogResult.OK) {
                OpenGearRom();
            } else {
                abortProgram = true;
            }
        }

        private void OpenGearRom() {
            this.rom = new GearRom(RomDialog.FileName);
        }

        private void ShowScreen() {
            GearScreenDisplay.BgPalette = currentPalette;
            GearScreenDisplay.Patterns = currentPatterns.PatternImage;
            GearScreenDisplay.Layout = currentScreen;
            GearScreenDisplay.Blocks = currentBlocks;


            GearScreenDisplay.RenderScreen();
        }


        PatternTable currentPatterns;
        int currentPatternsOffset;
        CompositePalette currentPalette;
        List<TileBlock> currentBlocks = new List<TileBlock>();
        ScreenLayout currentScreen;


        private void btnLoad_Click(object sender, EventArgs e) {
            int patternOffset = int.Parse(txtPatterns.Text, System.Globalization.NumberStyles.HexNumber);
            int paletteOffset = int.Parse(txtPalette.Text, System.Globalization.NumberStyles.HexNumber);
            int blockOffset = int.Parse(txtBlocks.Text, System.Globalization.NumberStyles.HexNumber);
            int screenOffset = int.Parse(txtLayout.Text, System.Globalization.NumberStyles.HexNumber);

            if (patternOffset != currentPatternsOffset) { // Don't reload patterns if they are the same
                if (currentPatterns != null) {
                    currentPatterns.Dispose();
                }

                currentPatterns = new PatternTable(true);
                currentPatterns.LoadWholeTable(rom, patternOffset);
                currentPatternsOffset = patternOffset;
            }

            currentPalette = new CompositePalette(rom, paletteOffset);
            currentScreen = new ScreenLayout(rom, (pHRom)screenOffset);

            currentBlocks.Clear();
            // 256 May be far, far more than needed
            for (int i = 0; i < 256; i++) {
                currentBlocks.Add(new TileBlock(rom, (pHRom)blockOffset + i * TileBlock.TileBlockDefSize));
            }

            ShowScreen();
        }






        protected override void OnVisibleChanged(EventArgs e) {
            base.OnVisibleChanged(e);

            if (Visible && abortProgram) {
                abortProgram = false;
                Close();
            }
        }

        private void GearScreenDisplay_BlockClicked(object sender, PointEventArgs e) {
            int blockIndex = currentScreen.GetBlockIndex(e.X, e.Y);   
            blockIndex++;
            currentScreen.SetBlockIndex(e.X, e.Y, (byte)blockIndex);

            ShowScreen();
        }

        private void GearScreenDisplay_BlockRightClicked(object sender, PointEventArgs e) {
            int blockIndex = currentScreen.GetBlockIndex(e.X, e.Y);
            blockIndex--;
            currentScreen.SetBlockIndex(e.X, e.Y, (byte)blockIndex);

            ShowScreen();

        }

        private void btnSave_Click(object sender, EventArgs e) {
            File.WriteAllBytes(RomDialog.FileName, rom.data);
        }
 
    }
}
