﻿using System;
using System.Collections.Generic;
using System.Text;
using Romulus.Nes;

namespace GearTool
{
    class GearScreenControl:ScreenControl
    {
        public GearScreenControl() 
        :base(){
            // Because flicker is annoying
            this.SetStyle(System.Windows.Forms.ControlStyles.OptimizedDoubleBuffer | System.Windows.Forms.ControlStyles.AllPaintingInWmPaint, true);
        }

        public event EventHandler<PointEventArgs> BlockClicked;
        public event EventHandler<PointEventArgs> BlockRightClicked;

        public ScreenLayout Layout { get; set; }
        public IList<TileBlock> Blocks { get; set; }
        public CompositePalette BgPalette { get; set; }



        protected override void ConstructNametable() {
            base.ConstructNametable();

            if (Layout == null || Blocks == null) return;

            // Loop over every tile block in the screen layout
            for (int blockX = 0; blockX < ScreenLayout.ScreenLayoutWidth; blockX++) {
                for (int blockY = 0; blockY < ScreenLayout.ScreenLayoutHeight; blockY++) {
                    int blockIndex = Layout.GetBlockIndex(blockX, blockY);

                    // Make sure the block we want to load exists
                    if (blockIndex < Blocks.Count) {
                        var block = Blocks[blockIndex];

                        // Loop over every tile in each block
                        for (int tileX = 0; tileX < TileBlock.TileBlockWidth; tileX++) {
                            for (int tileY = 0; tileY < TileBlock.TileBlockHeight; tileY++) {
                                int tileIndex = block.GetTile(tileX, tileY);

                                int screenX = blockX * TileBlock.TileBlockWidth + tileX;
                                int screenY = blockY * TileBlock.TileBlockHeight + tileY;
                                NameTable.SetTile(screenX, screenY, tileIndex);

                                // We are hard-coding a constant palette index of 1. Eventually,
                                // the attribute (palette index) data would need to be loaded from the ROM.
                                NameTable.SetPalette(screenX, screenY, 1);
                            }
                        }
                    }
                }
            }
        }

        protected override void LoadBgPalette(Romulus.ReturnEventArgs<CompositePalette> e) {
            base.LoadBgPalette(e);

            if (!BgPalette.IsEmpty)
                e.ReturnValue = BgPalette;
        }

        protected override void OnMouseDown(System.Windows.Forms.MouseEventArgs e) {
            base.OnMouseDown(e);

            int blockX = e.X / 32;
            int blockY = e.Y / 32;

            if (e.Button == System.Windows.Forms.MouseButtons.Left && BlockClicked != null) {
                BlockClicked(this, new PointEventArgs(blockX, blockY));
            } 
            if (e.Button == System.Windows.Forms.MouseButtons.Right && BlockRightClicked != null) {
                BlockRightClicked(this, new PointEventArgs(blockX, blockY));
            }
        }
        

    }
    class PointEventArgs : EventArgs
    {
        public PointEventArgs(int x, int y) {
            this.X = x;
            this.Y = y;
        }
        public int X { get; private set; }
        public int Y { get; private set; }
    }
    
}
